@@ -8,12 +8,14 @@
 #################################################################
 
 ##Author: Christos Zamantzas <christos.zamantzas@gmail.com>
-Version = '0.4-1'
+Version = '1.0-1'
 
 import os
 import sys
 import time
 
+import configCustom
+
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 from PyQt4.QtMaemo5 import *
@@ -21,61 +23,53 @@ from swappolubeUI import *
 
 ##############################################################
 
-Path         = '/home/user/.swappolube/'      # temp path of files
-AppPath      = '/opt/swappolube/'             # application path
-ConfigFile   = 'swappolube_config_temp'       # name of temp config file
-DefaultFile  = 'swappolube_config_default'    # name of default config file
-MainPath     = '/etc/event.d/'                # main path
-MainFile     = 'tuning'                       # tuning file
+PathTemp      = '/home/user/.swappolube'       # temp path of files
+PathApp       = '/opt/swappolube'              # application path
+PathSystem    = '/etc/event.d'                 # main path
+ConfigTemp    = 'swappolube_config_temp'       # name of temp config file
+ConfigCustom  = 'configCustom.py'              # name of custom config file
+ConfigMain    = 'tuning'                       # tuning file
 
 ##############################################################
 
-def CheckFile(Path, File):
+def CheckFile(Path, File):       
     ''' Check if a file exists. '''
 
     try:
-        f = open(Path + File, 'r')
+        f = open(Path + "/" + File, 'r')
         f.close()
     except:
         return False
     else:
         return True
 
-##############################################################
-        
-def showMessage(message):
-    ''' Method to display a message to the user that waits for a confirmation. '''
+def showMessage(message):        
+    ''' Method to display a message to the user that waits for an action. '''
 
     os.system('dbus-send --type=method_call --dest=org.freedesktop.Notifications \
                /org/freedesktop/Notifications \
                org.freedesktop.Notifications.SystemNoteDialog \
                string:"%s" uint32:0 string:"OK"' % message)          
 
-##############################################################  
-
-def showQuickMessage(message):
-    ''' Method to display a message to the user. '''
+def showQuickMessage(message):   
+    ''' Method to display shortly a message to the user. '''
 
     os.system('dbus-send --type=method_call --dest=org.freedesktop.Notifications \
                /org/freedesktop/Notifications \
                org.freedesktop.Notifications.SystemNoteInfoprint \
                string:"%s"' % message)          
 
-##############################################################  
-
-def doUpdateConfig():
+def doUpdateConfig():            
     ''' Update the config file with the edited. '''
 
     time.sleep(1)
     try:
-       os.system('sudo %supdateTuning copyTuning_temp' % AppPath)
+       os.system('sudo %s/updateTuning copyTuning_temp' % PathApp)
     except:
        return False
     else: 
        return True
        
-##############################################################
-
 class SwappolubeAbout(QtGui.QMainWindow):
     '''About Window'''
     def __init__(self, parent=None):
@@ -158,8 +152,6 @@ class SwappolubeAbout(QtGui.QMainWindow):
     def open_bugtracker(self):
         QDesktopServices.openUrl(QUrl('https://bugs.maemo.org'))
 
-##############################################################
-        
 class MyMainWindow(QtGui.QMainWindow):
         def __init__(self, parent=None):
         
@@ -167,8 +159,7 @@ class MyMainWindow(QtGui.QMainWindow):
              QtGui.QWidget.__init__(self, parent)
              self.ui = Ui_swappolube()
              self.ui.setupUi(self)
-             #new code
-             self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
+             ##self.setAttribute(Qt.WA_Maemo5AutoOrientation, True)
              self.setAttribute(Qt.WA_Maemo5StackedWindow, True)
              
              ##Connect the GUI Buttons with actions
@@ -177,6 +168,8 @@ class MyMainWindow(QtGui.QMainWindow):
              QtCore.QObject.connect(self.ui.btnApplyConfig, QtCore.SIGNAL('clicked()'), self.doApplyConfig)
              QtCore.QObject.connect(self.ui.btnSetCurrent, QtCore.SIGNAL('clicked()'), self.doSetCurrent)
              QtCore.QObject.connect(self.ui.btnSetProposed, QtCore.SIGNAL('clicked()'), self.doSetProposed)
+             QtCore.QObject.connect(self.ui.btnSetCustom, QtCore.SIGNAL('clicked()'), self.doSetCustom)
+             QtCore.QObject.connect(self.ui.btnSaveCustom, QtCore.SIGNAL('clicked()'), self.doSaveCustom)
              
                ##Connect Menu Buttons 
              QtCore.QObject.connect(self.ui.actionQuit, QtCore.SIGNAL('triggered()'), QtGui.qApp, QtCore.SLOT('quit()'))
@@ -190,13 +183,14 @@ class MyMainWindow(QtGui.QMainWindow):
         ##Create Methods
                 
         def doAbout(self):
+        
              stackwindow =SwappolubeAbout(self)
              stackwindow.show()
-             ##self.aboutWin = SwappolubeAbout(self)
 
         def doDeleteTunings(self):
-             os.system('sudo %supdateTuning deleteTuning' % AppPath)
-             if CheckFile(MainPath, MainFile) == True:
+        
+             os.system('sudo %s/updateTuning deleteTuning' % PathApp)
+             if CheckFile(PathSystem, ConfigMain) == True:
                 message = 'The tuning file could not be deleted.\
                            \nPlease delete it manually.'
              else: 
@@ -204,26 +198,26 @@ class MyMainWindow(QtGui.QMainWindow):
                            \nOn next reboot, the system will start with its default configuration.'
              showMessage(message)
 
-        def doRestoreDefault(self):
+        def doRestoreDefault(self): 
              ''' Apply the the default values.'''
              
              message = 'Please wait. Update in progress..'
              showQuickMessage(message)
              
              ##Update the values.
-             os.system('sudo %supdateTuning KillAllocatingTask 0' % AppPath )
-             os.system('sudo %supdateTuning PageCluster 5' % AppPath )
-             os.system('sudo %supdateTuning LaptopMode 0' % AppPath )
-             os.system('sudo %supdateTuning DirtyExpireCentisecs 500' % AppPath )
-             os.system('sudo %supdateTuning TCPTimestamps 1' % AppPath )
-             os.system('sudo %supdateTuning DirtyWritebackCentisecs 500' % AppPath )
-             os.system('sudo %supdateTuning TCPNoMetricsSave 0' % AppPath )
-             os.system('sudo %supdateTuning DirtyRatio 40' % AppPath )
-             os.system('sudo %supdateTuning Swappiness 100' % AppPath )
-             os.system('sudo %supdateTuning DirtyBGDRatio 10' % AppPath )
-             os.system('sudo %supdateTuning VFSCachePressure 100' % AppPath )
+             os.system('sudo %s/updateTuning KillAllocatingTask 0' % PathApp )
+             os.system('sudo %s/updateTuning PageCluster 5' % PathApp )
+             os.system('sudo %s/updateTuning LaptopMode 0' % PathApp )
+             os.system('sudo %s/updateTuning DirtyExpireCentisecs 500' % PathApp )
+             os.system('sudo %s/updateTuning TCPTimestamps 1' % PathApp )
+             os.system('sudo %s/updateTuning DirtyWritebackCentisecs 500' % PathApp )
+             os.system('sudo %s/updateTuning TCPNoMetricsSave 0' % PathApp )
+             os.system('sudo %s/updateTuning DirtyRatio 40' % PathApp )
+             os.system('sudo %s/updateTuning Swappiness 100' % PathApp )
+             os.system('sudo %s/updateTuning DirtyBGDRatio 10' % PathApp )
+             os.system('sudo %s/updateTuning VFSCachePressure 100' % PathApp )
              
-             if CheckFile(MainPath, MainFile) == True:
+             if CheckFile(PathSystem, ConfigMain) == True:
                 message = 'The default values have been restored to the system.\n\nWARNING: A tuning file exists and will reapply its settings on the next reboot.'
              else:   
                 message = 'The default values have been restored to the system.'
@@ -232,8 +226,114 @@ class MyMainWindow(QtGui.QMainWindow):
              ##Set current values in the GUI
              self.doSetCurrent()
              
-        def doSetCurrent(self): ##FIXME: separate reading and setting to two methods!!!
-             '''Set current values in the GUI'''
+        def doSaveCustom(self):     
+             ''' Save setting seen in the GUI as Custom Settings'''
+             
+             message = 'The custom settings are being saved please wait...'
+             showQuickMessage(message)
+             
+             ##Read GUI.
+             
+             radioSwappiness              = '0'
+             radioDirtyRatio              = '0'
+             radioDirtyBGDRatio           = '0'
+             radioDirtyExpireCentisecs    = '0'
+             radioDirtyWritebackCentisecs = '0'
+             radioPageCluster             = '0'
+             radioVFSCachePressure        = '0'
+             radioLaptopMode              = '0'
+             radioKillAllocatingTask      = '0'
+             radioTCPTimestamps           = '0'
+             radioTCPNoMetricsSave        = '0'
+
+             if self.ui.radioSwappiness.isChecked() == True:
+                radioSwappiness              = '1'
+             if self.ui.radioDirtyRatio.isChecked() == True:
+                radioDirtyRatio              = '1'
+             if self.ui.radioDirtyBGDRatio.isChecked() == True:
+                radioDirtyBGDRatio           = '1'
+             if self.ui.radioDirtyExpireCentisecs.isChecked() == True:
+                radioDirtyExpireCentisecs    = '1'
+             if self.ui.radioDirtyWritebackCentisecs.isChecked() == True:
+                radioDirtyWritebackCentisecs = '1'
+             if self.ui.radioPageCluster.isChecked() == True:
+                radioPageCluster             = '1'
+             if self.ui.radioVFSCachePressure.isChecked() == True:
+                radioVFSCachePressure        = '1'
+             if self.ui.radioLaptopMode.isChecked() == True:
+                radioLaptopMode              = '1'
+             if self.ui.radioKillAllocatingTask.isChecked() == True:
+                radioKillAllocatingTask      = '1'
+             if self.ui.radioTCPTimestamps.isChecked() == True:
+                radioTCPTimestamps           = '1'
+             if self.ui.radioTCPNoMetricsSave.isChecked() == True:
+                radioTCPNoMetricsSave        = '1'
+
+             
+             Swappiness                   = self.ui.lineSwappiness.text()
+             DirtyRatio                   = self.ui.lineDirtyRatio.text()
+             DirtyBGDRatio                = self.ui.lineDirtyBGDRatio.text()  
+             DirtyExpireCentisecs         = self.ui.lineDirtyExpireCentisecs.text()
+             DirtyWritebackCentisecs      = self.ui.lineDirtyWritebackCentisecs.text()
+             PageCluster                  = self.ui.linePageCluster.text()
+             VFSCachePressure             = self.ui.lineVFSCachePressure.text()
+             LaptopMode                   = self.ui.lineLaptopMode.text()
+             
+             
+             if  self.ui.checkBoxKillAllocatingTask.isChecked() == True:
+                KillAllocatingTask = '1'
+             else:
+                KillAllocatingTask = '0'
+             if self.ui.checkBoxTCPTimestamps.isChecked() == True:
+                TCPTimestamps = '1'
+             else:
+                TCPTimestamps = '0'
+             if self.ui.checkBoxTCPNoMetricsSave.isChecked() == True:
+                TCPNoMetricsSave = '1'
+             else:
+                TCPNoMetricsSave = '0'   
+                
+             ##Write values to the custom file.
+
+             os.system('echo "radioSwappiness = %s\
+                             \nSwappiness = %s\
+                             \nradioDirtyRatio = %s\
+                             \nDirtyRatio = %s\
+                             \nradioDirtyBGDRatio = %s\
+                             \nDirtyBGDRatio = %s\
+                             \nradioDirtyExpireCentisecs = %s\
+                             \nDirtyExpireCentisecs = %s\
+                             \nradioDirtyWritebackCentisecs = %s\
+                             \nDirtyWritebackCentisecs = %s\
+                             \nradioPageCluster = %s\
+                             \nPageCluster = %s\
+                             \nradioVFSCachePressure = %s\
+                             \nVFSCachePressure = %s\
+                             \nradioLaptopMode = %s\
+                             \nLaptopMode = %s\
+                             \nradioKillAllocatingTask = %s\
+                             \nKillAllocatingTask = %s\
+                             \nradioTCPTimestamps = %s\
+                             \nTCPTimestamps = %s\
+                             \nradioTCPNoMetricsSave = %s\
+                             \nTCPNoMetricsSave = %s" > %s/%s' % (  radioSwappiness, Swappiness, 
+                                                                    radioDirtyRatio,DirtyRatio,
+                                                                    radioDirtyBGDRatio, DirtyBGDRatio,
+                                                                    radioDirtyExpireCentisecs, DirtyExpireCentisecs,
+                                                                    radioDirtyWritebackCentisecs, DirtyWritebackCentisecs,
+                                                                    radioPageCluster, PageCluster,
+                                                                    radioVFSCachePressure, VFSCachePressure,
+                                                                    radioLaptopMode, LaptopMode,
+                                                                    radioKillAllocatingTask, KillAllocatingTask,
+                                                                    radioTCPTimestamps, TCPTimestamps,
+                                                                    radioTCPNoMetricsSave, TCPNoMetricsSave, 
+                                                                    PathApp, ConfigCustom ) )
+
+             message = 'The custom settings have been saved.'
+             showMessage(message)             
+             
+        def doSetCurrent(self):     
+             ''' Set CURRENT values in the GUI''' ##FIXME: split reading and setting into seperate methods!!!
              
              ##Set values for enable/disable   
              self.ui.radioKillAllocatingTask.setChecked(False)
@@ -283,12 +383,12 @@ class MyMainWindow(QtGui.QMainWindow):
              self.ui.linePageCluster.setText("%s" % PageCluster)
              self.ui.lineDirtyWritebackCentisecs.setText("%s" % DirtyWritebackCentisecs )
              self.ui.lineDirtyExpireCentisecs.setText("%s" % DirtyExpireCentisecs)
-
+             
              message = 'The CURRENT configuration options are shown.'
              showQuickMessage(message)
 
-        def doSetProposed(self): ##FIXME: outsourse proposed values to a file!!!
-             ''' Set proposed values in the GUI'''
+        def doSetProposed(self):    
+             ''' Set PROPOSED values in the GUI''' ##FIXME: outsource proposed values into a file!!!
              
              ##Set values for enable/disable   
              self.ui.radioKillAllocatingTask.setChecked(True)
@@ -320,8 +420,94 @@ class MyMainWindow(QtGui.QMainWindow):
 
              message = 'The PROPOSED configuration options are shown.'
              showQuickMessage(message)
+
+        def doSetCustom(self):      
+             ''' Set CUSTOM values read from a file'''
              
-        def doApplyConfig(self):
+             if CheckFile(PathApp, ConfigCustom) == False:
+                message = 'There is no custom file to be read.\
+                           \nYou need to create first one..'
+                showMessage(message)
+             else: 
+                message = 'The custom settings are being read please wait...'
+                showQuickMessage(message)
+                
+                ##load user custom variables
+                #import configCustom
+                reload(configCustom)
+
+                ##update the GUI
+                
+                #set first all checkboxes to False
+                self.ui.radioKillAllocatingTask.setChecked(False)
+                self.ui.radioPageCluster.setChecked(False)
+                self.ui.radioLaptopMode.setChecked(False)
+                self.ui.radioDirtyExpireCentisecs.setChecked(False)
+                self.ui.radioTCPTimestamps.setChecked(False)
+                self.ui.radioDirtyWritebackCentisecs.setChecked(False)
+                self.ui.radioTCPNoMetricsSave.setChecked(False)
+                self.ui.radioVFSCachePressure.setChecked(False)
+                self.ui.radioDirtyRatio.setChecked(False)
+                self.ui.radioSwappiness.setChecked(False)
+                self.ui.radioDirtyBGDRatio.setChecked(False)
+                
+                #now set those checkboxes=1 to True 
+                if configCustom.radioSwappiness == 1:
+                   self.ui.radioSwappiness.setChecked(True)
+                if configCustom.radioDirtyRatio == 1:
+                   self.ui.radioDirtyRatio.setChecked(True)
+                if configCustom.radioDirtyBGDRatio == 1:
+                   self.ui.radioDirtyBGDRatio.setChecked(True)
+                if configCustom.radioDirtyExpireCentisecs == 1:
+                   self.ui.radioDirtyExpireCentisecs.setChecked(True)
+                if configCustom.radioDirtyWritebackCentisecs == 1:
+                   self.ui.radioDirtyWritebackCentisecs.setChecked(True)
+                if configCustom.radioPageCluster == 1:
+                   self.ui.radioPageCluster.setChecked(True)
+                if configCustom.radioVFSCachePressure == 1:
+                   self.ui.radioVFSCachePressure.setChecked(True)
+                if configCustom.radioLaptopMode == 1:
+                   self.ui.radioLaptopMode.setChecked(True)
+                if configCustom.radioKillAllocatingTask == 1:
+                   self.ui.radioKillAllocatingTask.setChecked(True)
+                if configCustom.radioTCPTimestamps == 1:
+                   self.ui.radioTCPTimestamps.setChecked(True)
+                if configCustom.radioTCPNoMetricsSave == 1:
+                   self.ui.radioTCPNoMetricsSave.setChecked(True)                   
+                   
+                Swappiness                   = configCustom.Swappiness
+                DirtyRatio                   = configCustom.DirtyRatio
+                DirtyBGDRatio                = configCustom.DirtyBGDRatio  
+                DirtyExpireCentisecs         = configCustom.DirtyExpireCentisecs
+                DirtyWritebackCentisecs      = configCustom.DirtyWritebackCentisecs
+                PageCluster                  = configCustom.PageCluster
+                VFSCachePressure             = configCustom.VFSCachePressure
+                LaptopMode                   = configCustom.LaptopMode
+               
+                self.ui.lineSwappiness.setText("%s" % Swappiness)
+                self.ui.lineDirtyRatio.setText("%s" % DirtyRatio)
+                self.ui.lineDirtyBGDRatio.setText("%s" % DirtyBGDRatio )
+                self.ui.lineDirtyExpireCentisecs.setText("%s" % DirtyExpireCentisecs)
+                self.ui.lineDirtyWritebackCentisecs.setText("%s" % DirtyWritebackCentisecs)
+                self.ui.linePageCluster.setText("%s" % PageCluster)
+                self.ui.lineVFSCachePressure.setText("%s" % VFSCachePressure)
+                self.ui.lineLaptopMode.setText("%s" % LaptopMode)
+
+                self.ui.checkBoxKillAllocatingTask.setChecked(False)
+                self.ui.checkBoxTCPTimestamps.setChecked(False)
+                self.ui.checkBoxTCPNoMetricsSave.setChecked(False)
+ 
+                if configCustom.KillAllocatingTask == 1:
+                   self.ui.checkBoxKillAllocatingTask.setChecked(True)
+                if configCustom.TCPTimestamps == 1:
+                   self.ui.checkBoxTCPTimestamps.setChecked(True)
+                if configCustom.TCPNoMetricsSave == 1:
+                   self.ui.checkBoxTCPNoMetricsSave.setChecked(True)
+                   
+                message = 'The CUSTOM configuration options are shown.'
+                showQuickMessage(message)
+             
+        def doApplyConfig(self):    
              ''' Apply the configuration shown in the gui now! It will be lost on next reboot.'''
              
              message = 'Please wait. Update in progress..'
@@ -352,33 +538,35 @@ class MyMainWindow(QtGui.QMainWindow):
               
              ##Update the values.
              if self.ui.radioKillAllocatingTask.isChecked() == True:
-               os.system('sudo %supdateTuning KillAllocatingTask %s' % (AppPath, KillAllocatingTask) )
+               os.system('sudo %s/updateTuning KillAllocatingTask %s' % (PathApp, KillAllocatingTask) )
              if self.ui.radioPageCluster.isChecked() == True:
-               os.system('sudo %supdateTuning PageCluster %s' % (AppPath, PageCluster) )
+               os.system('sudo %s/updateTuning PageCluster %s' % (PathApp, PageCluster) )
              if self.ui.radioLaptopMode.isChecked() == True:
-               os.system('sudo %supdateTuning LaptopMode %s' % (AppPath, LaptopMode) )
+               os.system('sudo %s/updateTuning LaptopMode %s' % (PathApp, LaptopMode) )
              if self.ui.radioDirtyExpireCentisecs.isChecked() == True:
-               os.system('sudo %supdateTuning DirtyExpireCentisecs %s' % (AppPath, DirtyExpireCentisecs) )
+               os.system('sudo %s/updateTuning DirtyExpireCentisecs %s' % (PathApp, DirtyExpireCentisecs) )
              if self.ui.radioTCPTimestamps.isChecked() == True:
-               os.system('sudo %supdateTuning TCPTimestamps %s' % (AppPath, TCPTimestamps) )
+               os.system('sudo %s/updateTuning TCPTimestamps %s' % (PathApp, TCPTimestamps) )
              if self.ui.radioDirtyWritebackCentisecs.isChecked() == True:
-               os.system('sudo %supdateTuning DirtyWritebackCentisecs %s' % (AppPath, DirtyWritebackCentisecs) )
+               os.system('sudo %s/updateTuning DirtyWritebackCentisecs %s' % (PathApp, DirtyWritebackCentisecs) )
              if self.ui.radioTCPNoMetricsSave.isChecked() == True:
-               os.system('sudo %supdateTuning TCPNoMetricsSave %s' % (AppPath, TCPNoMetricsSave) )
+               os.system('sudo %s/updateTuning TCPNoMetricsSave %s' % (PathApp, TCPNoMetricsSave) )
              if self.ui.radioDirtyRatio.isChecked() == True:
-               os.system('sudo %supdateTuning DirtyRatio %s' % (AppPath, DirtyRatio) )
+               os.system('sudo %s/updateTuning DirtyRatio %s' % (PathApp, DirtyRatio) )
              if self.ui.radioSwappiness.isChecked() == True:
-               os.system('sudo %supdateTuning Swappiness %s' % (AppPath, Swappiness) )
+               os.system('sudo %s/updateTuning Swappiness %s' % (PathApp, Swappiness) )
              if self.ui.radioDirtyBGDRatio.isChecked() == True:
-               os.system('sudo %supdateTuning DirtyBGDRatio %s' % (AppPath, DirtyBGDRatio) )
+               os.system('sudo %s/updateTuning DirtyBGDRatio %s' % (PathApp, DirtyBGDRatio) )
              if self.ui.radioVFSCachePressure.isChecked() == True:
-               os.system('sudo %supdateTuning VFSCachePressure %s' % (AppPath, VFSCachePressure) )
+               os.system('sudo %s/updateTuning VFSCachePressure %s' % (PathApp, VFSCachePressure) )
 
              message = 'The new values have been applied to the system.\n\nINFO:They will revert to their defaults on the next reboot.\
                         \nIf you want them to persist use the Store button.'
              showMessage(message)
              
-        def doCreateConfig(self):
+        def doCreateConfig(self):   
+             ''' Create the tuning file'''
+             
              message = 'Please wait. Update in progress..'
              showQuickMessage(message)
 
@@ -406,9 +594,9 @@ class MyMainWindow(QtGui.QMainWindow):
                 TCPNoMetricsSave = '0'
             
              ##Update of the config file.
-             if CheckFile(Path, ConfigFile) == True:
+             if CheckFile(PathTemp, ConfigTemp) == True:
                 try:
-                   os.remove( Path + ConfigFile )
+                   os.remove( PathTemp + "/" + ConfigTemp )
                 except:
                    message = 'FATAL ERROR: The temp configuration file could not be removed.\nThe application will now exit..'
                    showMessage(message)
@@ -418,7 +606,7 @@ class MyMainWindow(QtGui.QMainWindow):
              else:
                 pass 
 
-             destination = open( Path + ConfigFile, "w" )
+             destination = open( PathTemp + "/" + ConfigTemp, "w" )
 
              destination.write('description "N900 tuning"\n')
              destination.write('author "swappolube editor"\n')
@@ -463,8 +651,6 @@ class MyMainWindow(QtGui.QMainWindow):
                 message = 'FATAL ERROR: The new configuration could not be created.'
                 showMessage(message)
             
-##############################################################
-             
 if __name__ == "__main__": 
         
           ##Check with what priviledges the GUI has been executed
@@ -474,8 +660,8 @@ if __name__ == "__main__":
              sys.exit(1)
           
           ##Check if temp folder is available   
-          if os.path.exists(Path) == False:
-             os.system('mkdir %s' % Path)
+          if os.path.exists(PathTemp) == False:
+             os.system('mkdir %s' % PathTemp)
           else:
              pass
        
